#ifndef ACCESS_HH
#define ACCESS_HH

//
// Klasse fuer eine Access-Object
// bestehend aus Subject und Liste von Permissions)
//

#include <iostream.h>
#include <fstream.h>
#include "../misc/list.hh"
#include "../misc/iterator.hh"
#include "permission.hh"




class TAccess
{
public:
  TAccess( int S ): Subject( S ) {
    Permissions = new TList<TPermission>; };

  TAccess(const TAccess& ac): Subject(ac.getSubject()) {     
    Permissions=new TList<TPermission>(ac.getPermissions()); };

  TAccess(ifstream &in)
  {
    int Zeilenlaenge=1024; // Maximale Zeilenlaenge betraegt 1024
    char puffer [ Zeilenlaenge ];    
    Permissions = new TList<TPermission>;
    for (int i=0;i<=1;i++)
      {
	in.getline(puffer,Zeilenlaenge);
	if (!strcmp(puffer,"PERMISSION"))
	  {
	    in.getline(puffer,Zeilenlaenge);
	    while (strcmp(puffer,"PERMISSIONEND"))
	      {		
		Perms P=READ;
		if (!strcmp(puffer,"READ"))  P=READ;
		if (!strcmp(puffer,"WRITE")) P=WRITE;
		if (!strcmp(puffer,"LOCK"))  P=LOCK;
		if (!strcmp(puffer,"APPEND"))P=APPEND;
		if (!strcmp(puffer,"OWN"))   P=OWN;
		TPermission Perm(P);		
		Permissions->addElement(Perm);		
		cerr << puffer << " PERM DETECTED" << endl;
		//cerr << *Permissions;
		in.getline(puffer,Zeilenlaenge);
	      }	    	   
	  }
	else sscanf(puffer,"%d",  &Subject);
      }
  }
    
  ~TAccess() { delete Permissions;};

  TAccess& TAccess::operator=(const TAccess &Ac)// Zuweisungsoperator
  {
    if (this == &Ac) return *this;
    Subject=Ac.getSubject();
    Permissions = new TList<TPermission>(Ac.getPermissions());
    return *this;
  }  

  friend bool operator ==( TAccess A1, TAccess A2 ) {     // Test auf Gleichheit
    return ( A1.getSubject() == A2.getSubject()
	     && A1.getPermissions() == A2.getPermissions()); };

  friend ostream& operator<<( ostream& os,TAccess &A ) {
    os << "ACCESS"           << endl 
       << A.getSubject()     << endl
       << "PERMISSION"       << endl
       << A.getPermissions() << "PERMISSIONEND" << endl;
    return os;
  }
  
  void addPermission(Perms P)
  {
    TPermission T(P);
    Permissions->addElement(T);
  }
  
  bool isIn(Perms P)
  {
    for (TIterator<TPermission> i(*Permissions); i.actualElement()!=0; i++)
      if (i.actual()==P) return true;
    return false;
  }

  bool delPerm(Perms P)
  {
    return Permissions->del(*(new TPermission(P)));
  }

  bool mutatePermission(Mod M, Perms P)
  {
    switch (M)
      {
      case ADD: if (isIn(P)) return false; else addPermission(P);return true;break;
      case DEL: if (isIn(P)) return delPerm(P); break;
      }   
    return false;
  }

  
  TList<TPermission>& getPermissions() const 
  { return *Permissions; };

  int getSubject() const 
  { return Subject; };
  
   
private:
  TList<TPermission> *Permissions;
  int Subject;
};

#endif //ACCESS_HH
