#ifndef ACL_HH
#define ACL_HH

//
// Klasse fuer eine Access-Control-List
// Liste von TAccess-Objecten (= Subject und Liste von Permissions)
//

#include <iostream.h>
#include <fstream.h>
#include "../misc/list.hh"
#include "permission.hh"
#include "access.hh"

class TAcl
{
public:
  TAcl( int Owner ) {
    TAccess OwnerAccess(Owner);
    OwnerAccess.addPermission(OWN);
    AccessList = new TList<TAccess>;
    AccessList->addElement(OwnerAccess);
  };
  

  TAcl(const TAcl& acl): AccessList(new TList<TAccess>(acl.getList()))  {};
  
  TAcl(ifstream &in)
  {
    int Zeilenlaenge=1024; // Maximale Zeilenlaenge betraegt 1024
    char puffer [ Zeilenlaenge ];
    AccessList = new TList<TAccess>;
    in.getline(puffer,Zeilenlaenge);
    while(strcmp(puffer,"ACLEND"))
      {
	if (!strcmp(puffer,"ACCESS")) 
	  {
	    TAccess Acc(in);
	    AccessList->addElement(Acc);	    	    
	  }	
	in.getline(puffer,Zeilenlaenge);		
      }
  }
    
  ~TAcl() { delete AccessList;};

  TAcl& TAcl::operator = ( const TAcl &A)// Zuweisungsoperator
  {
    if (this == &A) return *this;
    AccessList=new TList<TAccess>(A.getList());
    return *this;
  }

  friend bool operator==( TAcl A1, TAcl A2 ) {     // Test auf Gleichheit
    return A1.getList() == A2.getList(); };

  friend ostream& operator<<( ostream& os, TAcl &A ) {
    os << "ACL" << endl
       << A.getList()
       << "ACLEND" << endl;
	return os;
  }
  
  TList<TAccess>& getList() const { return *AccessList; };

  // Besitzer der AccessControlListe
  int getOwner()
  {
    for (TIterator<TAccess> i(*AccessList); i.actualElement()!=0; i++)
      {	
	if ((i.actual()).isIn(OWN))
	  return (i.actual()).getSubject();
      }
   return 0;   
  }

  // Ist der Kunde bereits in der AccessList ?
  bool isIn(int K) {
    for (TIterator<TAccess> i(*AccessList); i.actualElement()!=0; i++)
      {	
	if (K==(i.actual()).getSubject())
	  return true;
      }   
    return false;
  }

  // Hat der Kunde irgendwelche Rechte ?
  bool hasAnyPermission(int K)
  {
    if (! isIn(K)) return false;
    else // Sind Rechte in der Liste ?
      {
	printf("%d\n",!((getAccessData(K)).getPermissions()).isEmpty());
      return !(((getAccessData(K)).getPermissions()).isEmpty());
      }
    return false;	    
  }


  // Liefert die Permissions eines Kunden
  TAccess& getAccessData(int K)
  {
    for (TIterator<TAccess> i(*AccessList); i.actualElement()!=0; i++)
      {	
	if (K==(i.actual()).getSubject())
	  return i.actual();
      }  
   TAccess *buffer = new TAccess(0);	
   return *buffer;
  }


  // Addiert oder Loescht Permissions aus der Liste  
  bool mutateAccess(Mod M,Perms P,int K) 
  {
    if (isIn(K)) 
      {
	bool b=(getAccessData(K)).mutatePermission(M,P);
	// Hat der Kunde noch Rechte, falls nein Kunden aus der Liste loeschen	
	if ( ( ! hasAnyPermission(K) ) && b )	   
	    AccessList->del(getAccessData(K));
	return b;
      }
    else if (M==ADD)
      {
	TAccess Buffer(K);
	Buffer.addPermission(P);
	AccessList->addElement(Buffer);
	return true;
      }    
    return false;
  }
  
   
private:
  TList<TAccess> *AccessList;
};

#endif //ACL_HH


