#ifndef __BANK_HH
#define __BANK_HH
//
//  Project   : besy
//  File      : bank.hh
//  Author    : Ronald Kriemann / Christian Rost
//  Purpose   : class for a bankserver
//  Date      : 25.05.1997
//  Changes   : 
//


#include <unistd.h>
#include "../misc/defines.hh"
#include "../misc/list.hh"
#include "../misc/md5.hh"
#include "../misc/dynarray.hh"

#include "../misc/message.hh"
#include "konto.hh"
#include "kunde.hh"
#include "transaktion.hh"
#include "TCoinData.hh"

//
// class for the bankserver 
// - has all the data
// - communicates with kommuclients via message queues
//
class Bank 
{
protected:
	// message queue id
	int  msqid;

	// the data
	TList<TKonto> *KontoListe;
	TList<TKunde> *KundenListe;

	// MD5
	TMD5 md5; 	

	// MD5-Result
	uchar digest_md5[17];

    // DB for coins
    TDynArray< TCoinData >  arrCoins;
    
public:
	// constructor
	Bank ();
	// destructor
	~Bank ();

	// Result = base64( MD5(input) )
	TString  encrypt ( const TString &input );
  
	// Checkt LoginInformation
	bool isAuthorized ( int nummer, const TString &name, 
						const TString &pass, int login = 0);

	// Erzeugt ein neues Konto
	bool createKonto ( int Nummer, TKunde *kunde, double Betrag=0);

	// Erzeugt ein neuen Kunden
	bool createKunde ( int Nummer, 
					   const TString &Name,
					   const TString &passw, 
					   const TString &OnePass,
					   const TString &key);

	// Gibt Kunden zurueck
	TKunde* getKunde ( int Nummer);
	TKunde* getKunde ( const TString &);

	// Gibt Konto zurueck
	TKonto* getKonto ( int Nummer);

	// Loescht ein vorhandenes Konto
	bool deleteKonto ( int Nummer);

	// Fuegt eine Transaktion hinzu
	bool addTransaktion ( int Source, int Target, double Betrag,
						  const TString &Zweck, int Modus);

	// Gibt Kontostand zurueck
	double getKontoStand ( int Nummer);

	// Gibt Liste mit Transaktionen zurueck
	TList<TTransaktion>* getAktionen ( int Nummer);

	// Aendert AccessControlliste eines Kontos
	bool mutateAccessControl ( int Nummer);  

	// sends a string to the commu-server with correct pid
	void sendString ( pid_t pid, const TString &str, int more=0 );
	void sendString ( pid_t pid, double value);

	// handles client-requests
	void handleClientRequest ( pid_t pid, MSGData &data);

	// handles kommu with kommuservers
	int run ();

	// Funktion zum einmaligen Erstellen der Konten und Kunden        
	void initBankdata();

  
	// saves all bank-data to file
	bool save(const TString &file);
  
	// loads bank-data from file
	bool load(const TString &file);

    // sign and return given coin
    TString signCoin ( const TString & string );
    
    //
    // access coin DB
    //
    bool hasSerialNo ( long serialNo ) const;

  // check the coin if its already spended and returns a message
  TString spendCoin(  int , const TString & string );

    // output
	friend ostream& operator << ( ostream& os, Bank &K );
};

#endif  // __BANK_HH

