// Datei: client.cc

extern "C"
{
#include "unistd.h"
}

#include "../misc/TRandom.hh"
#include "../misc/TBitArray.hh"
#include "../misc/base64.hh"

#include "client.hh"
#include "gui_konto.hh"
#include "gui_login.hh"
#include "gui_coin.hh"

// some defines for signing a coin
#define NOTHING          0
#define BEGIN_SIGN       1
#define CREATE_NEW_COIN  2
#define SEND_COIN        3
#define RECV_KEYREQ      4
#define SEND_KEYS        5
#define RECV_SIGNED_COIN 6
//#define SEND_BLINDFAC    7

// some defines for transmitting a coin
#define NOTHING           0
#define STARTTRANSMITCOIN 1
#define TRANSMITCOIN      2
#define TRANSMITID        3


// defines for rsa import
#define CLIENT  0
#define BANK    1
#define COIN    2
#define STORE   3

#define PRIVATE 0
#define PUBLIC  1

//
// constructor
//

TClient *Client;

TClient::TClient()
{  
    setCaption("Client");
    gui_login *loginTab = new gui_login(this,"page_one");    
    gui_konto *kontoTab = new gui_konto(this,"page_two");
    gui_coin  *coinTab  = new gui_coin(this,"page_three");
    addTab( loginTab, "Login" );
    addTab( kontoTab, "Konto" );
    addTab( coinTab, "Coin" ); 
    
    Client=this;    
    msgData.setKundenNr(14);          // Delete this
    msgData.setKundenname("besy14");  //    "     "
       
    out = new Tout(0,"");
    out->show();
    setOkButton ( "QUIT" );
    reply=0;         //msg_queue is empty
    bIDEA=false;     //IDEA-En/Decryption en/disabled      
    srand(getpid());
    

    rsaClient.setVerbosity( 1 );
    rsaBank.setVerbosity( 1 );

    // RSA ist ausgeschaltet und wird manuell ueber den RSA-Button aktiviert
    rsaState = 0;

    connect(this,SIGNAL(applyButtonPressed()),qApp,SLOT(quit()));
  
    for (int i = 0; i < NO_OF_COIN; i++)
    {
        arrCoin[i] = NULL;
        arrKey[i].setSize( KEY_SIZE );
    }// for
    usedCoin = -1;
    signCoinState = NOTHING;           
    transmitCoinState=NOTHING;

    serverPort = 3014;
}



TClient::~TClient()
{    
    if (Socket.connected())
    {
        send(Socket,"ENDE\n");
        Socket.close();  
        out->appendstring("Disconnected from Server");
    }// if
    if (storeSocket.connected())
    {        
        storeSocket.close();  
        out->appendstring("Disconnected from Store");
    }// if

    if (out)
        delete out;

    for (int i = 0; i < NO_OF_COIN; i++)
    {
        if (arrCoin[i] != NULL)
            delete arrCoin[i];
    }// for
}


void
TClient::connectServer(const char* serverName,int serverPort)
{
    int   sd;
    
    // first disconnect
    if (Socket.connected())
    {
        send( Socket,"ENDE\n" );
        Socket.close();  
        out->appendstring( "Disconnected from Server" );
        bIDEA = false;
        delete sn;
    }// if

    out->appendstring("-------------Connect to Server------------------");    
    
    Socket.setPort(serverPort);
    if ((sd = Socket.connect(serverName)) >= 0)
    {
        out->appendstring("Connected");
        sn = new QSocketNotifier(sd, QSocketNotifier::Read);
        connect( sn, SIGNAL(activated(int)), this, SLOT(dataReceived(int)));
    }// if
}



void
TClient::dataReceived(int s)
{  
    TString socketmsg;
                          
    if (s==Socket.getDescriptor())   
    {
        if (Socket.recvln(socketmsg) < 0)
            Socket.close();
    }
    else
	{
        if (storeSocket.recvln(socketmsg) < 0)
            storeSocket.close();
	}	

    if (socketmsg.length()) // Nachricht vorhanden ?
    {                      
        //socketmsg enthaelt nun die erhaltene Nachricht\n           
        //Abschneiden des letzten \n
        socketmsg.setLength(socketmsg.length()-1); 	

        // RSA
        if (rsaState > 0 && rsaState <= 3)
        {
            if (socketmsg.substr(0, 4) == "FAIL")
                rsaState = NOTHING;
            else
                rsaLogin(socketmsg);
        }// if
        else
        {
            if (bIDEA && s!=storeSocket.getDescriptor())
                socketmsg=idea.decode(TBase64::decode(socketmsg));
            if (signCoinState != NOTHING)
                signCoin( socketmsg );
            else
            {
                if (transmitCoinState != NOTHING)
                    sendCoin( socketmsg );
                else
                {
                    char tmp[socketmsg.length()+1];
                    socketmsg.copyTo(tmp);
                    tmp[socketmsg.length()]=0;
                    out->appendstring(tmp);
		  
                    //msg_queue weiter abarbeiten
                    if (reply==1)           		  
                    {
                        if (s==Socket.getDescriptor())   
                            sendmsg(Socket);
                        else
                            sendmsg(storeSocket);
                    }		 
                }// else
            }// else
        }    
    }
}



//Schickt eine Message zum Bankserver/ Geschaeft
//Falls diese aus mehreren Zeilen besteht,
//werden diese Zeilen einzeln abgeschickt
//und es wird jeweils auf Antwort vom 
//Bankserver gewartet
void
TClient::sendmsg(TSocket &Sock, const char* string=0)
{ 
    int lines=0;
    char *msg=0;
    if ( string != 0 || strlen(string)!=0 )    
    {
        msg=(char*) malloc(strlen(string)+1);
        msg_queue=(char*) malloc(strlen(string)+1);
        strncpy(msg,string,strlen(string));
        msg[strlen(string)]=0;       
    }
    else if (msg_queue != 0)
    {
        msg=(char*) malloc(strlen(msg_queue)+1);
        strncpy(msg,msg_queue,strlen(msg_queue));
        msg[strlen(msg_queue)]=0;
        if (strlen(msg)==0) 
        {
            reply=0;       
            delete msg_queue;      
        }     
    }  

    if (strlen(msg)) //Ueberhaupt was da ?     
    {
        for (unsigned int i=0; i <= strlen(msg); i++)   
            if (msg[i] == '\n')  // Ende der Zeile
            {
                lines++;
                if (lines==1) // Ende der 1. Zeile
                {
                    char head[i+2];          
                    head[i+1]=0;
                    strncpy(head,msg,i+1);        
                    //msg_queue mit dem Rest fuellen
                    strncpy(msg_queue,&msg[i+1],strlen(msg)-i-1); 
                    msg_queue[strlen(msg)-i-1]=0;                
                    // 1. Zeile wegschicken
                    reply=1; 
                    // Warten auf Antwort vom Server, um die msg_queue weiter abzuarbeiten          
                    // 1. Zeile zum Server schicken
                    send(Sock,head);
                }     
            }   
        delete msg;  
    }
}
 

void
TClient::send( TSocket &Sock, const TString &msg )
{
    TString buffer;

    if (Sock.connected())
    {
        if (bIDEA && &Sock!=&storeSocket)
            Sock.sendln( TBase64::encode( idea.encode( msg )) );
        else 
            Sock.send(msg);
    }
}



// Operation requested:

void 
TClient::login(const char* password)
{
    if ( ! Socket.connected() )
    {
        cout << "TClient::login (...) : not connected" << endl;
        return;
    }// if
    
    out->appendstring("-------------------Login------------------------");
    msgData.setOperation(LOGIN);
    msgData.setKundenPwd(password);
    sendmsg(Socket,msg.buildClientMsg(msgData));
};

void 
TClient::onelogin(const char* password,int md5Factor)
{
    if ( ! Socket.connected() )
    {
        cout << "TClient::onelogin (...) : not connected" << endl;
        return;
    }// if
    
    out->appendstring("----------------Onetimelogin--------------------"); 
    msgData.setOperation(ONELOGIN);
    msgData.setKundenPwd(password);
    
    //Calculate Pw
    uchar digest[17];
    uchar digest2[17];
    uchar digestbase[60]; 
    digest[17]=0;
    digest2[17]=0;    
    md5.crypt(digest,(uchar*)msgData.getKundenPwd(),strlen(msgData.getKundenPwd()));
    for (int i=1;i<md5Factor;i++)
    {              
        memcpy((void *)digest2,(void *)digest,16);      
        md5.crypt(digest,digest2,16);                                                
	
    }
    TBase64::encode(digestbase,digest,16);
    printf("BASE64(MD5**%d(%s))= [%s] \n",md5Factor,msgData.getKundenPwd(),digestbase);
    msgData.setKundenPwd((char*)digestbase);
    //end

    sendmsg(Socket,msg.buildClientMsg(msgData));
  
};

void
TClient::doRSA(int Port)
{
    if ( ! Socket.connected() )
    {
        cout << "TClient::doRSA (...) : not connected" << endl;
        return;
    }// if
    
    rsaState=1;
    serverPort=Port;
    rsaLogin("");
}

void 
TClient::rsaLogin(const TString &socketmsg )
{
    int         nounce1, nounce2;
    TString     crypted, decrypted;
    char        string[256];
    idea_key_t  ideakey;           
    int         size;
    uchar       buffer[16];
    TBitArray   key;
    
    bIDEA = false;

    if (rsaState == 1)
    {
        if (!loadKey( CLIENT, msgData.getKundenNr(), PRIVATE ) ||
            !loadKey( CLIENT, msgData.getKundenNr(), PUBLIC  ) ||
            !loadKey( BANK, serverPort - 3000, PUBLIC ))
        {
            cerr << "TClient::rsaLogin () : could not load all needed keys" << endl;
            rsaState = 0;
            return;
        }// if
    }// if
    
    switch (rsaState)
    {
        case 1 :
            //
            // Sending Kundennr and Nounce1
            //

            out->appendstring( "-----------------RSA-LOGIN----------------------" );
            out->appendstring( "PHASE I" ); 

            rsaNounce = TRandom::randomInt();

            cout << "TClient::rsaLogin (...) : sending kdnr = " << msgData.getKundenNr() << endl
                 << "                          and  nounce1 = " << rsaNounce << endl;
            
            sprintf( string, "BESY%d,%d", msgData.getKundenNr(), rsaNounce);
            Socket.sendln( TBase64::encode( rsaBank.encryptPublic( string ) ) );

            break;      

        case 2 :
            //
            // receiving nounce1, nounce2 and sending IDEA-key
            //
            out->appendstring("PHASE II"); 
            
            decrypted = rsaClient.decryptPrivate( TBase64::decode( socketmsg ) );
            
            nounce1 = nounce2 = -1;
            decrypted.strtok( ',' ) >> nounce1;
            decrypted >> nounce2;
            
            // checking nounce
            if (rsaNounce == nounce1)
            {
                cout << "TClient::rsaLogin (...) : got correct nounce1 = " << nounce1 << endl
                     << "                          reading     nounce2 = " << nounce2 << endl;

                key.setSize( 1024 );
                key.initRandom();
                idea.generateSubKeys( key.toString() );
                idea.exportKey( ideakey );
                memcpy( buffer, ideakey, 16 );
                
                cout << "TClient::rsaLogin (...) : sending IDEA-KEY = ";
                for (int i=0; i < 16 ; i++)
                    cout << (uint) buffer[i] << ' ';
                cout << endl;

                sprintf(string,"%d,",nounce2);
                size = strlen(string);
                memcpy( &string[size], ideakey, 16 );
                size += 16;
                string[size] = 0;

                crypted  = rsaClient.sign( TString(string,size) );
                cout << "TClient::rsaLogin (...) : length = " << crypted.length() << endl;
                cout << "TClient::rsaLogin (...) : signed = " << crypted << endl;
                crypted  = TBase64::encode( rsaBank.encryptPublic( crypted ) );
                crypted += "\n";

                send(Socket,crypted);
            }// if
            else
            {
                cout << "TClient::rsaLogin (...) : wrong nounce1 = " << nounce1
                     << " was given" << endl;
                out->appendstring("RSA-LOGIN failed (Nounce-check failed)");
                send( Socket,"FAIL" );
                rsaState = NOTHING;
            }// else
            break;

        case 3 : // Receiving FAIL or OK from Server   
            out->appendstring("PHASE III");

            if (socketmsg=="OK") 
            {
                cout << "TClient::rsaLogin (...) : secure IDEA-connection established" << endl;
                out->appendstring("RSA-LOGIN successfull");
                bIDEA = TRUE;
            }// if
            else
            {
                cout << "TClient::rsaLogin (...) : IDEA-connection failed" << endl;
                out->appendstring("RSA-LOGIN failed");
                rsaState = NOTHING;
            }// else
            
            break;      

        default :
            rsaState = NOTHING;
            break;
    }// switch

    if (rsaState != NOTHING)
        rsaState++;
};

//
// loads modulus and private key from file
//
bool
TClient::loadKey ( int owner, int number, int type )
{
    FILE  *file;
    char   buffer[16384];
    int    n;
    char   key[16384];

    switch (type)
    {
        case PRIVATE : file = fopen( "../keys/private_keys", "r" ); break;
        case PUBLIC  : file = fopen( "../keys/public_keys", "r" );  break;
        default        : return FALSE;
    }// switch

    if (file == NULL)
    {
        cerr << "Server::loadKey (...) : could not open file" << endl;
        return FALSE;
    }// if

    while (!feof( file ))
    {
        fgets( buffer, sizeof(buffer), file );
        
        memset( key, 0, sizeof(key) );
        n = -1;
        
        switch (owner)
        {
            case CLIENT : sscanf( buffer, "K_BESY%d,%s", &n, (char*) &key ); break;
            case BANK   : sscanf( buffer, "B_BESY%d,%s", &n, (char*) &key ); break;
//             case COIN   : sscanf( buffer, "C_BESY%d,%s", &n, (char*) &key ); break;
//             case STORE  : sscanf( buffer, "S_BESY%d,%s", &n, (char*) &key ); break;
            default       :
                fclose( file );
                return FALSE;
        }// switch

        if (n == number)
        {
            if (type == PRIVATE)
            {
                switch (owner)
                {
                    case CLIENT : rsaClient.setPrivateKey( TBase64::decode( key ) ); break;
                    case BANK   : rsaBank.setPrivateKey( TBase64::decode( key ) ); break;
//                     case COIN   : rsaCoin.setPrivateKey( TBase64::decode( key ) ); break;
//                     case STORE  : rsaStore.setPrivateKey( TBase64::decode( key ) ); break;
                }// switch
            }// if
            else // type == PUBLIC
            {
                switch (owner)
                {
                    case CLIENT : rsaClient.setModulus( TBase64::decode( key ) ); break;
                    case BANK   : rsaBank.setModulus( TBase64::decode( key ) ); break;
//                     case COIN   : rsaCoin.setModulus( TBase64::decode( key ) ); break;
//                     case STORE  : rsaStore.setModulus( TBase64::decode( key ) ); break;
                }// switch
            }// else

            fclose( file );
            
            return TRUE;
        }// if
    }// while

    fclose( file );

    return FALSE;
}

void 
TClient::kstand()
{
    if ( ! Socket.connected() )
    {
        cout << "TClient::kstand (...) : not connected" << endl;
        return;
    }// if
    
    out->appendstring("-----------------Kontostand---------------------");
    msgData.setOperation(KSTAND);
    sendmsg(Socket,msg.buildClientMsg(msgData));    
}
  

void 
TClient::kausz() 
{
    if ( ! Socket.connected() )
    {
        cout << "TClient::kausz (...) : not connected" << endl;
        return;
    }// if
    
    out->appendstring("-----------------Kontoauszug---------------------");
    msgData.setOperation(KAUSZ);
    sendmsg(Socket,msg.buildClientMsg(msgData));
}

void 
TClient::ueberw()
{
    if ( ! Socket.connected() )
    {
        cout << "TClient::ueberw (...) : not connected" << endl;
        return;
    }// if
    
    out->appendstring("-----------------Ueberweisen---------------------");
    msgData.setOperation(UEBERW);
    sendmsg(Socket,msg.buildClientMsg(msgData));
}

void
TClient::recht()
{
    if ( ! Socket.connected() )
    {
        cout << "TClient::recht (...) : not connected" << endl;
        return;
    }// if
    
    out->appendstring("----------------Rechte ndern---------------------");
    msgData.setOperation(RECHTE);
    sendmsg(Socket,msg.buildClientMsg(msgData));
}


void 
TClient::connect_to_store(const char *name,int port)
{
    int  sd;    
    // first disconnect
    if (storeSocket.connected())
    {      
        storeSocket.close();  
        out->appendstring( "Disconnected from Store" );        
        delete storesn;
	
    }// if

    out->appendstring("-------------Connect to Store------------------");    
    
    storeSocket.setPort(port);
    if ((sd = storeSocket.connect(name)) >= 0)
    {
        out->appendstring("Connected");
        storesn = new QSocketNotifier(sd, QSocketNotifier::Read);
        connect( storesn, SIGNAL(activated(int)), this, SLOT(dataReceived(int)));
    }// if 
    transmitCoinState = NOTHING;
}


void 
TClient::buy()
{  
    if ( ! storeSocket.connected() )
    {
        cout << "TClient::buy (...) : not connected" << endl;
        return;
    }// if
    
    sendCoin("");
}

void 
TClient::getcoin()
{
    if ( ! Socket.connected() )
    {
        cout << "TClient::getcoin (...) : not connected" << endl;
        return;
    }// if
    
    signCoinState = BEGIN_SIGN;
    signCoin( "" );
}



//////////////////////////////////////////////////////////////
//
// methods for DigiCash
//

void
TClient::signCoin ( const TString &socketMsg )
{
    TString  string;
    int      i;
    char     szClient[64], szBank[64];
    TIdea    coinIdea;
    
    TBigInteger<RSA_BASE>  bnBlind, bnHash;
    
    static   int  coinPos;
    static   int  k;
    
    // encrypted coins
    static TString   arrStrCoin[NO_OF_COIN];

    if (signCoinState == SEND_COIN)
    {
        // check if msg was recieved
        if (socketMsg[0] != 'O')
        {
            cout << "TClient::signCoin (...) : coins did get through to bank" << endl;
            signCoinState = NOTHING;
            return;
        }// if

        if ((coinPos % 2) == 0)
        {
            // send coin
            cout << "TClient::signCoin (...) : sending coin "
                 << coinPos / 2 << " to bank" << endl;

            send( Socket, TBase64::encode( arrStrCoin[coinPos/2] ));
        }// if
        else
        {
            // send md5(coin)
            cout << "TClient::signCoin (...) : sending md5(coin) "
                 << coinPos / 2 << " to bank" << endl;

            bnBlind = TBigInteger<RSA_BASE>( arrKey[coinPos/2].toString() );
            bnBlind.powerMod( rsaBank.getPublicKey(), rsaBank.getModulus() );
            
            bnHash  = TBigInteger<RSA_BASE>( md5.crypt( arrCoin[coinPos/2]->getCoinStr() ) );
            bnHash *= bnBlind;
            bnHash %= rsaBank.getModulus();
            
            send( Socket, TBase64::encode( bnHash.toString() ) );;
        }// else
        
        coinPos++;
        if (coinPos == (2*NO_OF_COIN))
            signCoinState++;

        return;
    }// if
    else if (signCoinState == SEND_KEYS)
    {
        if (socketMsg[0] != 'O')
        {
            cout << "TClient::signCoin (...) : keys did get through to bank" << endl;
            signCoinState = NOTHING;
            return;
        }// if

        if (coinPos == k)
            coinPos++;

        if (coinPos >= NO_OF_COIN)
            signCoinState++;
        
        if (coinPos < NO_OF_COIN)
        {
            string = TBase64::encode( arrKey[coinPos].toString() );

            // send first key to server
            cout << "TClient::signCoin (...) : sending key ( " << coinPos
                 << " ), length = " << string.length() << endl;
            send( Socket, string );

            coinPos++;

            return;
        }// if
    }// else
    
    switch (signCoinState)
    {
        case BEGIN_SIGN :
            msgData.setOperation( SIGNCOIN );
            msgData.setCoinStr( "Start" );
            send( Socket, msg.buildClientMsg( msgData ) );
            break;
            
        case CREATE_NEW_COIN :
            //
            // create new coin (with predefined value)
            //

            if (socketMsg[0] != 'O')
            {
                cout << "TClient::signCoin (...) : sever not ready" << endl;
                signCoinState = NOTHING;
                return;
            }// if
            
            cout << "TClient::signCoin (...) : creating new coins" << endl;

            // delete old coins
            pCoin = NULL;
            for (i = 0; i < NO_OF_COIN; i++)
            {
                if (arrCoin[i] != NULL)
                    delete arrCoin[i];
            }// for

            sprintf( szClient, "K_BESY%d", msgData.getKundenNr() );
            sprintf( szBank  , "B_BESY%d", serverPort - 3000 );

            for (i = 0; i < NO_OF_COIN; i++)
            {
                arrCoin[i] = new TCoin( 10, szClient );
                arrCoin[i]->prepare( szBank );

                arrKey[i].setSize( 128 );
                arrKey[i].initRandom();

                // encrypt coins
                coinIdea.generateSubKeys( arrKey[i].toString() );
                arrStrCoin[i] = coinIdea.encode( arrCoin[i]->getCoinStr() );
            }// for

            cout << "TClient::signCoin (...) : sending coin (0) to bank" << endl;
            send( Socket, TBase64::encode( arrStrCoin[0] ) );

            coinPos = 1;

            break;

        case RECV_KEYREQ :
            //
            // get chosen number and send all blindfactors except the k'th
            //

            if (socketMsg[0] != 'O')
            {
                signCoinState = NOTHING;
                cout << "TClient::signCoin (...) : error while transmitting coindata" << endl;
                return;
            }// if

            string = socketMsg;
            string <<= 3;

            k = -1;
            string >> k;
            
            if ((k >= 0) && (k < NO_OF_COIN))
            {
                cout << "TClient::signCoin (...) : got key request ( k = " << k << " )" << endl;

                if (k == 0)
                    coinPos = 1;
                else
                    coinPos = 0;
                
                string = TBase64::encode( arrKey[coinPos].toString() );
                // send first key to server
                cout << "TClient::signCoin (...) : sending key ( " << coinPos
                     << " ), length = " << string.length() << endl;
                send( Socket, string );

                coinPos++;
            }// if
            else
            {
                signCoinState = NOTHING;
                cout << "TClient::signCoin (...) : did not get a correct k = "
                     << k << endl;
            }// else
            
            break;

        case RECV_SIGNED_COIN :
            //
            // look if coin was signed and save it
            //

            string = socketMsg;
            
            if ((string[0] == 'O') && (string[1] = 'K'))
            {
                cout << "TClient::signCoin (...) : coin was signed" << endl;
                out->appendstring("Coin received");
                string <<= 3;
                if (string[string.length()-1] == '\n')
                    string.setLength( string.length() - 1 );
                bnBlind = TBigInteger<RSA_BASE>( arrKey[k].toString() );
                bnHash  = TBigInteger<RSA_BASE>( TBase64::decode( string ) );
                bnHash *= bnBlind.invMod( rsaBank.getModulus() );
                pCoin = arrCoin[k];
                pCoin->setSignedCoin( bnHash.toString() );

            }// if
            else
            {
                cout << "TClient::signCoin (...) : error while signing coin" << endl;
            }// else
            
            signCoinState = NOTHING;
            
        default:
            signCoinState = NOTHING;
            break;
    }// switch

    if (signCoinState != NOTHING)
        signCoinState++;
}


void
TClient::sendCoin(const TString &socketMsg)
{
    TString buffer,subStr;
    TString temp(socketMsg);

    if ( pCoin != NULL && storeSocket.connected() )
    {
        switch (transmitCoinState)
        {	  
            case NOTHING :
                msgData.setOperation( SIGNCOIN );
                msgData.setCoinStr("START");
                send( storeSocket, msg.buildClientMsg( msgData ) );	
                transmitCoinState++;	        
                break;
                
            case STARTTRANSMITCOIN :	  	
                if (socketMsg[0] == 'O')
                {
                    cout << "TClient::sendCoin (...) : transmitting signed coin : "
                         << pCoin->getCoinStr().substr(0, 30) << endl;
                    cout << "TClient::sendCoin (...) : transmitting md5(coin)" << endl;

                    buffer=pCoin->getCoinStr();
                    buffer+="|";
                    buffer+=TBase64::encode(pCoin->getSignedCoin());
                    buffer+=".";
                    msgData.setOperation( SIGNCOIN );
                    msgData.setCoinStr( buffer );
                    send( storeSocket, msg.buildClientMsg( msgData ) );
                    transmitCoinState++;	        
                }
                else
                    transmitCoinState=NOTHING;
                break;
                
            case TRANSMITCOIN :	  
                if (socketMsg[0] != 'O')
                {
                    cout << "TClient::sendCoin (...) : store does'nt accept our coin "
                         << "[Wrong Signature]" << endl;
                    transmitCoinState = NOTHING;
                    return;
                }// if

                cout << "TClient::sendCoin (...) : following ID's requested: ";
                subStr  = temp.strtok( '(' );
                buffer="(";
                for (int i = 0; i < NO_OF_ID; i++)
                {
                    int value;
                    subStr  = temp.strtok( ',' ) ;
                    subStr >> value;
                    cout << value << ",";
                    buffer+=TBase64::encode((pCoin->getBlindFac(i,value)).toString());
                    buffer+=",";
                }// for
                buffer+=")";
                cout << endl;
                msgData.setOperation( SIGNCOIN );
                msgData.setCoinStr( buffer );
                send( storeSocket, msg.buildClientMsg( msgData ) );
                transmitCoinState=NOTHING;
                
            default:
                transmitCoinState=NOTHING;	      
                break;	  	  
        }
    }// if
    else
    {
        out->appendstring("No coins available or not connected to a store");
        transmitCoinState=NOTHING;
    }// else
}
