#ifndef __SERVER_HH
#define __SERVER_HH
//
//  Project   : besy
//  File      : server.hh
//  Author    : Ronald Kriemann / Christian Rost
//  Purpose   : class for a socketserver
//  Date      : 
//  Changes   : 
//


#include "../misc/defines.hh"
#include "../misc/socket.hh"           // Socketcommunication

#include "../misc/msgdata.hh"          // Messagequeue Communication 
#include "../misc/message.hh"          // with banksrv


#include "../misc/idea.hh"     // Some En/Decryption-Stuff
#include "../misc/TRSA.hh"
#include "../misc/TCoin.hh"

//
// this class opens a socket at given portnumber
// if a client has connected, the process will be forked
// and the methode run will be called
//
class Server 
{
protected:
	// Socket, wich the server is using
	TSocket Socket;  

	int bIDEA, bRSA;
	int        nRSAState,nRSANounce;
	int        rsaKNr;
  
	TIdea   idea;
	TRSA    rsaBank, rsaClient, rsaCoin, rsaStore;

    // data for signing coins
    TString  arrCoinStr[NO_OF_COIN];
    TString  arrCoinMD5[NO_OF_COIN];
    int      signCoinState;
    
	// msqid for the commu with the bankserver
	int  msqid;

	// are we a forked child
	bool bForked;

public:
	// constructors
	Server ( int portnumber );
	// destructors
	~Server ();
  
	// init the socket
	void init ();
  
	// wait for clients
	int wait ();
  
	// do your job
	int run (int nsd);
  
	// do parsing of messages
	bool parseMsg ( int nsd, TString plainMsg, MSGData &msgData, bool &bEndCommu );

	// sends the string to a socket
	bool sendln(int sd, const TString & string); 

	// sends a message to bank 
	// returns the reply of the bank
	TString sndMsg(TMessage &msg);

	void rsaLogin(int nsd, const TString & socketmsg, MSGData &msgData);

	// loads modulus and private key from file
    bool loadKey ( int owner, int number, int type );

    //
    // methods for DigiCash
    //
    void signCoin ( int clientSocket, const TString &socketMsg );
    bool checkCoin ( const TString & server );
    bool hasSerialNo ( int serialNo );
};

#endif  // __SERVER_HH
