//
//  Project   : libmisc
//  File      : TClientSocket.cc
//  Author    : Ronald Kriemann
//  Purpose   : class for socket-based communication
//

extern "C" {
#include <stdio.h>
#include <unistd.h>
#include <netdb.h>
#include <string.h>

#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>

#include <netinet/in.h>
}

#include "TClientSocket.hh"

//
// constructor
//
TClientSocket::TClientSocket ( uint port )
{
	serverName = "";
	bConnected = FALSE;

	setPort( port );
	initSocket();
}

TClientSocket::TClientSocket ( uint port, const TString &name )
{
	serverName = name;
	bConnected = FALSE;

	setPort( port );
	initSocket();

	cout << "host : " << serverName << endl;
}

//
// connect to socket
//
int 
TClientSocket::connect ()
{
	struct sockaddr_in name;
	int                len;
	char               buf[256];
	struct hostent    *pHostent;

	if ((sd >= 0) && (!bConnected))
	{
		// apply servername
		if (serverName == "")
		{
			// get local hostname
			if (gethostname(buf, sizeof(buf)) < 0)
			{
				perror( "TClientSocket::connect () : gethostname" );
				return ERROR;
			}// if
		}// if
		else
			serverName.copyTo( buf, sizeof( buf ) );

		cout << "host : " << buf << endl;

		// look up host's network adress
		if ((pHostent = gethostbyname(buf)) == NULL)
		{
			perror( "TClientSocket::connect () : gethostbyname" );
			return ERROR;
		}// if
		
		// create the address of the server
		memset( &name, 0, sizeof( struct sockaddr_in ));
		
		name.sin_family = AF_INET;
		name.sin_port   = htons(getPort());
		memcpy( &name.sin_addr, pHostent->h_addr_list[0], pHostent->h_length );
		
		len = sizeof( struct sockaddr_in );
		
		// connect to the server
		if (::connect(sd, (struct sockaddr *) &name, len) < 0)
		{
			perror( "TClientSocket::connect () : connect" );
			return ERROR;
		}// if
		else
			bConnected = TRUE;
	}// if

	return ERROR;
}
