//
//  Project   : libmisc
//  File      : TCoin.cc
//  Author    : Ronald Kriemann
//  Purpose   : class for a digital coin
//

extern "C"
{
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
}

#include "md5.hh"
#include "base64.hh"
#include "TRandom.hh"

#include "TCoin.hh"

//
// constructor
//
TCoin::TCoin ( const TFloat v, const TString &strID )
{
    int  i;
    
	value = v;
	id    = strID;
    id.setSize( ID_SIZE );

    for (i = 0; i < NO_OF_ID; i++)
    {
        arrID[i][0].setSize( 0 );
        arrID[i][1].setSize( 0 );
    }// for

    serialNo = -1;
}

TCoin::TCoin ( const TString &s )
{
	signedCoin = s;
}

//
// prepare unsigned coin for signing by bank
//
TString 
TCoin::prepare ( const TString &bankID )
{
    ulong    i,j;

    TMD5     md5;
	TString  string;
    char     buf[256];

    TBitArray  temp;
    TBitArray  arrR[NO_OF_ID][2];
    TBitArray  arrP[NO_OF_ID][2];

    //
    // create components von secret splitting
    //

    for (i = 0; i < NO_OF_ID; i++)
    {
        arrID[i][0].setSize( ID_SIZE );
        arrID[i][0].initRandom();

        arrID[i][1]  = id;
        arrID[i][1] ^= arrID[i][0];
    }// for

    //
    // init bitarrays for bit-commitment-protocol
    //
    for (i = 0; i < NO_OF_ID; i++)
    {
        arrR[i][0].setSize( BIT_COMMIT_SIZE );
        arrR[i][0].initRandom();
            
        arrR[i][1].setSize( BIT_COMMIT_SIZE );
        arrR[i][1].initRandom();
    }// for

    for (i = 0; i < NO_OF_ID; i++)
    {
        for (j = 0; j < 2; j++)
        {
            string  = arrR[i][j].toString();
            string += arrID[i][j].toString();
            string  = md5.crypt( string );

//             cout << "TCoin::prepare (...) : md5(blind[" << i << ',' << j << "]) = "
//                  << TBase64::encode( string ) << endl;
//             cout << "TCoin::prepare (...) :         R[" << i << ',' << j << "]) = "
//                  << TBase64::encode( arrR[i][j].toString() ) << endl;
            
            string += arrR[i][j].toString();

            arrP[i][j] = string;
        }// for
    }// for

    //
    // build coin as string
    //
    sprintf( buf, "%.2f", value );
    string = buf;

    string += ",";
    string += bankID;
    
    serialNo = buildNextSerialNo();
    sprintf( buf, ",%ld", serialNo );
    string += buf;

    for (i = 0; i < NO_OF_ID; i++)
    {
        string += ",(";
        string += TBase64::encode( arrP[i][0].toString() );
        string += ",";
        string += TBase64::encode( arrP[i][1].toString() );
        string += ")";
    }// for
    
	return szCoin = string;
}

//
// access i'th ID-bitstring (index 0 or 1)
//
const TBitArray &
TCoin::getBlindFac( const uint &i, const uint &j ) const
{
    if ((i < NO_OF_ID) && (j < 2))
        return arrID[i][j];
    else
    {
        cerr << "TCoin::getBlindFac (" << i << ',' << j << ") : index out of range" << endl;
        exit( ERROR );
    }// else
}

//
// access signed coin
//
const TString &
TCoin::getSignedCoin () const
{
    return signedCoin;
}

TCoin &
TCoin::setSignedCoin ( const TString & string )
{
    signedCoin = string;
    
    return *this;
}

//
// access coin string
//
const TString &
TCoin::getCoinStr () const
{
    return szCoin;
}

//
// copy operator
//
TCoin &
TCoin::operator = ( const TCoin & coin )
{
	uint  i;
    
	id         = coin.id;
	value      = coin.value;
    szCoin     = coin.szCoin;
    signedCoin = coin.signedCoin;

    for (i = 0; i < NO_OF_ID; i++)
    {
        arrID[i][0] = coin.arrID[i][0];
        arrID[i][1] = coin.arrID[i][1];
    }// for

    return *this;
}
    
//
// build next serial no.
//
long
TCoin::buildNextSerialNo ()
{
    return TRandom::randomInt( 0, RAND_MAX );
}
