#ifndef __TEXCEPTION_HH
#define __TEXCEPTION_HH
//
//  Project  : libmisc
//  File     : TException.hh
//  Author   : Ronald Kriemann
//  Purpose  : baseclass for all exceptions and some
//             commonly used subclasses
//

extern "C" {
#include <sys/types.h>
}

#include <iostream.h>

#include "defines.hh"

//
// baseclass for all exception
// - if a system-wide baseclass would be defined
//   set it as ancestor of TException
//
class TException
{
public:
    // constructor
    TException ();

    // destructor
    virtual ~TException ();

    // output
    virtual void print ( ostream & ) const;
};

//
// output (operator)
//
ostream & operator << ( ostream &, const TException & );

//
// exception for range-overflow
//
class TRange : public TException
{
protected:
    // wanted pos and upper bound
    size_t   pos, max;

    // indicate if data was given
    bool     bPos, bMax;
    
public:
    // constructor
    TRange ();
    TRange ( size_t p );
    TRange ( size_t p, size_t m );

    // access pos and bound
    virtual const size_t getPos () const;
    virtual const size_t getMax () const;
    
    // output
    virtual void print ( ostream & ) const;
};

//
// exception for wring sizes
//
class TSize : public TException
{
protected:
    // wrong size
    size_t  size;

    // indicate whether size was given or not
    bool    bSize;

public:
    // constructor
    TSize ();
    TSize ( size_t s );

    // access size
    virtual const size_t getSize () const;

    // output
    virtual void print ( ostream & ) const;
};

//
// baseclass for mathematical exceptions
//
class TMathErr : public TException
{
public:
    // output
    virtual void print ( ostream & ) const;
};

//
// class for division by zero exception
//
class TDivByZero : public TMathErr
{
public:
    // output
    virtual void print ( ostream & ) const;
};

//
// class for arithmetic underflow exceptions
//
class TUnderFlow : public TMathErr
{
public:
    // output
    virtual void print ( ostream & ) const;
};

#endif  // __TEXCEPTION_HH
