#ifndef __TRSA_HH
#define __TRSA_HH
//
//  Project   : besy
//  File      : TRSA.hh
//  Author    : Ronald Kriemann
//  Purpose   : class for RSA encoding and decoding
//

#include "TBigInteger.hh"
#include "TString.hh"

// base for TBigInteger
#define RSA_BASE  0x8000

// keysize
#define KEY_SIZE  1024

//
// class for RSA en- and decoding
//
class TRSA
{
protected:
	// verbositylevel
	int  verbosity;

	// public-, privatekey and modulus
	TBigInteger<RSA_BASE>   e, d, n;

public:
	// constructor
	TRSA ();

	// destructor
	~TRSA ();

	// access keys
	TBigInteger<RSA_BASE> & getPrivateKey ();
	TBigInteger<RSA_BASE> & getPublicKey ();
	TBigInteger<RSA_BASE> & getModulus ();
	void setPrivateKey ( const TBigInteger<RSA_BASE> &k );
	void setPublicKey ( const TBigInteger<RSA_BASE> &k );
	void setModulus ( const TBigInteger<RSA_BASE> &k );

	// build keys
	void buildKeys ();

	// en-, decrypt and sign a message
	TString encryptPrivate ( const TString &input ) const;
	TString encryptPublic ( const TString &input ) const;
	TString encrypt ( const TBigInteger<RSA_BASE> &k, const TString &input ) const;

	TString decryptPrivate ( const TString &input ) const;
	TString decryptPublic ( const TString &input ) const;
	TString decrypt ( const TBigInteger<RSA_BASE> &k, const TString &input ) const;

	TString sign ( const TString &input ) const;

    // check a signed msg (assuming : sign at end)
    bool checkSign ( const TString & signedMsg ) const;

    // remove sign from msg (assuming : sign at end)
    void removeSign ( TString & msg ) const;
    
    // return fingerprint of public key
    TString fingerPrint () const;
    
	// set verbosity level
	void setVerbosity ( int n );

	// output
	friend ostream & operator << ( ostream &os, TRSA &rsa );
};

#endif  // __TRSA_HH
