#ifndef __idea_HH
#define __idea_HH

//  Project   : libmisc
//  File      : idea.hh
//  Author    : Christian Rost
//  Purpose   : class for idea encoding and decoding
//

#include <stdio.h>
#include <string.h>
#include "TString.hh"
#include "md5.hh"
#include "defines.hh"

#define IDEA_ROUNDS 8                            /* Number of IDEA rounds  */
#define IDEA_SK_NUM (6 * IDEA_ROUNDS + 4)        /* Number of IDEA subkeys */

typedef ushort idea_key_t[8];                    /* local key type */
typedef ushort idea_subkeys_t[IDEA_SK_NUM];      /* local IDEA subkeys type */
typedef ushort idea_block_t[4];                  /* in/output IDEA data block */

class TIdea
{
public:
    // Constructor
    TIdea ();
    // Destructor
    ~TIdea ();

    //create the set of subkeys for encryption
    void encrypt_subkeys(idea_key_t,idea_subkeys_t);
    //create the set of subkeys for decryption  
    void decrypt_subkeys(idea_subkeys_t,idea_subkeys_t);

    //Idea Block cipher (encryption AND decryption)
    void cipher(idea_block_t inblock,
                idea_block_t outblock,
                idea_subkeys_t); //set of subkeys generated by en/de-crypt_subkeys

    //delete subkey-set from memory
    void clearkey();

    //creates encryption and decryption subkeys
    void generateSubKeys(const TString & text = "");

    //the following stuff needs md5
    //buildEncryptKeys creates the encryption key-set with an md5(text)-key
    void buildEncryptKeys( const TString & text );
    
    //buildDecryptKeys creates the decryption key-set with an md5(text)-key
    void buildDecryptKeys( const TString & text );
    
    //crypts an  text-block "in" to the text-block "out"
    // !Danger!: If decryption fails (caused by bad key) then "out" contains "bad" ascii-code
    void cypher_textblock(uchar out[8],
                          uchar in[8]);
  
    void cypher_text(int modus, char* out, const char* in, int inlen=0);

    void encode(char* out, const char* in, int length);

    void decode(char* out, const char* in, int inlen);

    TString encode(const TString &in);
  
    TString decode(const TString &in);

    void exportKey(idea_key_t);

    void importKey(idea_key_t);

private:
    //Multiplication modulo 2**16 + 1^M
    ushort mul(ushort x , ushort y);

    //Compute multiplicative inverse of x by Euclid's GCD algorithm
    ushort mul_inv(ushort x);

    int modus; // 0=encrypt 1=decrypt
  
    idea_subkeys_t enc_subkeys;
    idea_subkeys_t de_subkeys;

    idea_key_t main_key;
  

};

#endif //__idea_HH

