#ifndef LIST_HH
#define LIST_HH

#include <iostream.h>

// forward decl
template <class T> class TIterator;

//
// Allgemeine Klasse fuer eine ueber
// Pointer verkettete Liste
//

// Elemente der Liste

template <class Type>
class TElement
{
public:
 // TElement() { next=0; };	
  TElement(const Type &D, TElement *E=0):Data(D), next(E) { };    // Konstruktor
  bool isEqual(TElement<Type> E){return E.Data==Data;}           // Wann sind zwei Elemente gleich ?
  Type Data;                                                      // Datenfeld
  TElement<Type> *next;                                           // Naechstes Element
};
    


// Liste mit Methoden zum Verwalten der Liste

template <class Type>
class TList
{
  friend class TIterator<Type>;
    
public:  
  TList(): Element(0),last(0) {};           // Konstruiert eine leere Liste
  TList( const TList<Type> &L )             // Copy-Konstruktor  
  {  
    Element=0;last=0;
    TElement<Type> *iterator=L.Element;
    while ( iterator!=0  ) 
      { addElement(iterator->Data);
      iterator=iterator->next;
      }	
  };
  ~TList() {del();};                       // Zerstoert eine Liste
  
  // Fuegt an die Liste ein Element hinzu
  void addElement(const TElement<Type> &E) {
    // Neues Element erstellen
    TElement<Type> *NewElement = new TElement<Type>(E);
    // Anhaengen
    if (isEmpty()) { Element = NewElement;}
    else  last->next = NewElement;
    // last aktualisieren
    last = NewElement;
  }

  /*   void addElement(TElement<Type> *E) {
    // Anhaengen
    if (isEmpty()) { Element = E;}
    else  last->next = E;
    // last aktualisieren
    last = E;
  }
  */
  
  
  friend ostream&
  operator<<( ostream& os, TList<Type> &List )
  {    
    TElement<Type> *buffer = List.Element;
    while ( buffer !=0 && buffer != List.last)
      {
	os << buffer->Data;
	buffer=buffer->next;
      }
    if (buffer !=0) 
      os << buffer->Data; 
    return os;
  }
  
  friend bool operator ==( TList<Type> L1, TList<Type> L2 ) {     // Test auf Gleichheit
    TElement<Type> *buffer = L1.Element;
    bool in = true;
    while ( buffer !=0 && in==true)
      {
	in = L2.isIn(buffer->Data);
	buffer=buffer->next;
      }
    if (in==false) return false;

    buffer = L2.Element;
    while ( buffer !=0 && in==true)
      {
	in = L1.isIn(buffer->Data);
	buffer=buffer->next;
      }
    return in;
  };
  


  // Check ob Element in der Liste
  bool isIn( TElement<Type> E ) {
    // Falls leer dann nix drin
    if ( isEmpty() ) return false;
    // Erstes und letztes Element checken
    if ( Element->isEqual(E) || last->isEqual(E) ) return true;
    // Liste durchlaufen
    if (Element!=last)
      {
	TElement<Type> *buffer = Element->next;
	for ( ; buffer != last; buffer = buffer->next )
	  {
	    if ( buffer->isEqual(E) ) return true;
	  }
	return false;
      }
    return false;
  }
  
  // Ckeck ob Liste leer
  bool isEmpty() {return Element == 0 ? true : false;}
  
  int del( TElement<Type> E ){   // Entfernt ein Element 
    TElement<Type> *buffer = Element;
    int i=0;
    
    // Checken ob E am Anfang in der Liste
    while ( buffer !=0 && buffer->isEqual(E) )
      {
	// Falls ja  dann entfernen
	TElement<Type> *temp = buffer->next;
	delete buffer;
	i++;
	buffer=temp;      
      }  
    
    // Die Ersten Elemente sind nun nicht E
    if ( (Element=buffer)==0 ) // Check ob Liste leer
      {
	last=0;
	return i;
      }  
    
    TElement<Type> *prev = buffer; 
    buffer = buffer->next;
    // Weiter in der Liste
    while ( buffer )
      {
	if (buffer->isEqual(E))
	  {
	    prev->next = buffer->next;
	    if (last == buffer)
	      last = prev;
	    delete buffer;
	    i++;
	    buffer = prev->next;
	  }
	else
	  {
	    prev=buffer;
	    buffer=buffer->next;
	  }      
      }
    return i;
  } 
  
  //private:

  void del(){                          // Alles entfernen
    // Liste durchlaufen und alle 
    // Speicherplaetze wieder freigeben
    TElement<Type> *buffer = Element;
    while (buffer !=0)
      {
	TElement<Type> *temp = buffer;
	buffer = buffer->next;
	delete temp;
      }
    // Liste ist nun leer
    Element=last=0;
  }
  
  
  TElement<Type> *Element;             // Erstes Element in der Liste
  TElement<Type> *last;                // Letztes Element in der Liste
  
};

#endif // LIST_HH





