//
//  Project   : besy
//  File      : message.hh
//  Author    : Ronald Kriemann / Christian Rost
//  Purpose   : class for a message between commu- and bank-server
//  Date      : 04.06.1997
//  Changes   : 
//

#include "message.hh"

//
// some needed types
//
struct ElementData
{
    int     nData;
    double  fData;
    char   *sData;
};

//
// Konstruktor
//
TMessage::TMessage ()
{ 
    pData = new MSGData; 
    limiter='\x0B'; // Vertikaler Tab als Trennzeichen       
}

//
// destructor
//
TMessage::~TMessage()
{
    delete pData;
}

//
// Baut String-Message aus den Daten
// Fuer die Bank -> Commuserver-Verbindung
//
char*
TMessage::buildBankMsg(MSGData &data) 
{ 	  		 
    sizeOfMsg=0;
    memset( buffer,0,sizeof(buffer));
    
    // Wir bauen den String (buffer) auf
    switch (data.getOperation())
    {
	    case RSA :
            addData(data.getOperation());
            addData(data.getKundenNr());
            break;
	    case LOGIN :
	    case ONELOGIN : // Login mit Onetimepasswort
            addData(data.getOperation());	    
            addData(data.getKundenNr());
            addData(data.getKundenname());	      
            addData(data.getKundenPwd());
            break;
	    case KAUSZ :
	    case KSTAND : 
            addData(data.getOperation());	      
            addData(data.getKundenNr());
            addData(data.getKundenname());
            addData(data.getKundenPwd());
            addData(data.getKontoNr());	                    	      
            break;
	    case UEBERW :
            addData(data.getOperation());
            addData(data.getKundenNr());
            addData(data.getKundenname());
            addData(data.getKundenPwd());
            addData(data.getKontoNr());
            addData(data.getZielKontoNr());
            addData(data.getZielname());
            addData(data.getBetrag());
            addData(data.getZweck());	      
            break;
	    case RECHTE :
            addData(data.getOperation());	    
            addData(data.getKundenNr());
            addData(data.getKundenname());	      
            addData(data.getKundenPwd());
            addData(data.getKontoNr());
            addData(data.getZielnummer());
            addData(data.getRechtAbheben());
            addData(data.getRechtKontoauszug());
            break;
	    case SIGNCOIN :
            addData( data.getOperation() );
            addData( data.getCoinStr() );
            break;
    case SPENDCOIN :      
      addData(data.getOperation());
      addData(data.getSerialNo());
      addData(data.getCoinIDs());
      break;      
        case CHECKSERIAL :
            addData( data.getOperation() );
            addData( data.getSerialNo() );
            break;    
	    case NOOP :	     
	    default :	
            addData(data.getOperation());
            break;
    }	     	   	  
    return buffer;	  
}

//
// Baut String-Message aus den Daten
// Fuer die Client -> Commuserver-Verbindung
//
char*
TMessage::buildClientMsg ( MSGData &data ) 
{
    static char  string[16384];
    
    TString tmp;
    sizeOfMsg = 0;
    memset( buffer,0,sizeof(buffer));
    string[0] = 0;
    
    // Wir bauen den String (buffer) auf
    switch (data.getOperation())
    {
	    case LOGIN :
            sprintf(buffer,
                    "KUNDENNR %d\nNAME %s\nPASSWORT %s\n",
                    data.getKundenNr(),
                    data.getKundenname(),
                    data.getKundenPwd());
            break;

	    case ONELOGIN : // Login mit Onetimepasswort
            sprintf(buffer,
                    "KUNDENNR %d\nNAME %s\nONETIMEPW %s\n",
                    data.getKundenNr(),
                    data.getKundenname(),
                    data.getKundenPwd());
            break;

	    case KAUSZ :
            sprintf(buffer,
                    "KTNR %d\nOP KAUSZ\n",
                    data.getKontoNr());
            break;
		     
	    case KSTAND : 
            sprintf(buffer,
                    "KTNR %d\nOP KSTAND\n",
                    data.getKontoNr());
            break;

	    case UEBERW :
            sprintf(buffer,
                    "KTNR %d\nOP UEBERW\nZKTNR %d\nZNAME %s\nBETRAG %f\nZWECK %s\n",
                    data.getKontoNr(),
                    data.getZielKontoNr(),
                    data.getZielname(),
                    data.getBetrag(),
                    data.getZweck());	      
            break;

	    case RECHTE :
            sprintf(buffer,
                    "KTNR %d\nOP RECHTE\nZNR %d\n",
                    data.getKontoNr(),
                    data.getZielnummer());

            if (data.getRechtAbheben())
                strcat(buffer,"+ ABHEBEN\n");
            else
                strcat(buffer,"- ABHEBEN\n");

            if (data.getRechtKontoauszug())
                strcat(buffer,"+ KONTOAUSZUG\n.\n");
            else
                strcat(buffer,"- KONTOAUSZUG\n.\n");
            
            break;

        case SIGNCOIN :
            data.getCoinStr().copyTo( string, sizeof(string) );
            string[data.getCoinStr().length()] = 0;
            sprintf( buffer,
                     "OP SIGNCOIN %s\n",
                     string );
            break;       
    case CHECKSERIAL :
      sprintf( buffer,
                     "OP CHECKSERIAL %d\n",
                     data.getSerialNo() );
            break;

	    case NOOP :
	    default : // Nothing to do
            break;
    }	     	   	  
    return buffer;	  
}

char*
TMessage::getMsg()
{
    return buffer;
}

//
// Haengt ein int-Wert an den Datenstring
//
void
TMessage::addData(int data)
{ 
    char temp[20];    
    int  size;

    size = sprintf( temp, "%c1 %d", limiter, data );
    strcat( buffer, temp );
    sizeOfMsg += size;
}

//
// Haengt ein float-Wert an den Datenstring
//
void
TMessage::addData(double data)
{
    char temp[20];
    int  size;

    size = sprintf( temp, "%c2 %f", limiter, data );
    strcat( buffer, temp );
    sizeOfMsg += size;
} 

//
// Haengt ein String an den Datenstring
//
void
TMessage::addData(const char* data)
{
    char *temp;
    int   size;

    temp = (char*) malloc(strlen(data)+1);
    size = sprintf( temp, "%c3 %s", limiter, data );
    strcat( buffer, temp );
    sizeOfMsg += size;
    
    delete temp;
} 


//
// Haengt ein String (TString) an den Datenstring
//
void
TMessage::addData( const TString &data )
{
    char *temp;

    temp = (char*) malloc( data.length()+1 );
    data.copyTo( temp, data.length()+1 );

    addData( temp );
    delete temp;
} 


MSGData&
TMessage::getData(const char* msg)
{    
    int elements=0;unsigned int i=0;
    // Anzahl der Dateneintrage feststellen    
    while (i<strlen(msg))
    {
        if (msg[i]==limiter) elements++;    
        i++;
    }    
    return *(extractData(elements,msg));
}
    
MSGData*
TMessage::extractData(int elements, const char* msg)
{
    ElementData element[elements];
    int type;int e=0;int size;  
    
    // Erstmal den String auslesen
    for (unsigned int i=0; i < strlen(msg);i++)
    {
        if (msg[i]==limiter)  // Nachfolgendes Daten-Element lesen
        { 
            i++;
            sscanf(&msg[i],"%d",&type);
            // Der Typ des nachfolgenden DatenElements steht fest	 
            i++;i++; element[e].sData=0;
            switch (type)
            {
                case 1 :
                    //int
                    sscanf(&msg[i],"%d",&(element[e].nData));	      
                    break;
                case 2 :
                    //double	      
                    sscanf(&msg[i],"%lf",&(element[e].fData));
                    break;
                case 3 :
                    //string
                    size = getLength(&msg[i]);
                    element[e].sData = (char *) malloc(size);    	 
                    strncpy(element[e].sData,&msg[i],size);	  
                    element[e].sData[size]=0;	      
                    break;	      
                case 4 : break;
		    // TString     
            }
            e++;	  
        }// if
    }// for

    // Alle Daten sind bekannt
    // und werden nun entsprechend der ausgewaehlten Operation in pData gespeichert
    
  
    // Erstes Feld ist die OperationsID
    pData->setOperation((Operation)element[0].nData);
    switch (pData->getOperation())
    {
       case SPENDCOIN :
	 pData->setSerialNo(element[1].nData);	 
	 pData->setCoinIDs((TString) element[2].sData);
	 delete (element[2].sData);
	 break;
	 
        case SIGNCOIN :
            pData->setCoinStr(element[1].sData);
            delete (element[1].sData);
            break;

        case CHECKSERIAL :
            pData->setSerialNo( element[1].nData );
            break;
            
        case RSA :
            pData->setKundenNr(element[1].nData);
            break;

        case ONELOGIN : // Login mit OnetimePasswort
        case LOGIN :
            pData->setKundenNr(element[1].nData);
            pData->setKundenname(element[2].sData);
            pData->setKundenPwd(element[3].sData);
            //strings aufraeumen
            delete (element[2].sData);
            delete (element[3].sData);
            break;

        case KAUSZ :	
        case KSTAND :	 	
            pData->setKundenNr(element[1].nData);
            pData->setKundenname(element[2].sData);
            pData->setKundenPwd(element[3].sData);
            pData->setKontoNr(element[4].nData);
            //strings aufraeumen
            delete (element[2].sData);
            delete (element[3].sData);
            break;

        case UEBERW :        
            pData->setKundenNr(element[1].nData);
            pData->setKundenname(element[2].sData);
            pData->setKundenPwd(element[3].sData);
            pData->setKontoNr(element[4].nData);
            pData->setZielKontoNr(element[5].nData);
            pData->setZielname(element[6].sData);
            pData->setBetrag(element[7].fData);
            pData->setZweck(element[8].sData);
            //strings aufraeumen
            delete (element[2].sData);
            delete (element[3].sData);
            delete (element[8].sData);
            break;

        case RECHTE :
            pData->setKundenNr(element[1].nData);
            pData->setKundenname(element[2].sData);
            pData->setKundenPwd(element[3].sData);
            pData->setKontoNr(element[4].nData);
            pData->setZielnummer(element[5].nData);
            pData->setRechtAbheben(element[6].nData);
            pData->setRechtKontoauszug(element[7].nData);
            //strings aufraeumen
            delete (element[2].sData);
            delete (element[3].sData);     	
            break;

        case NOOP :
        default : // Nothing to do
            break;
    }	         
    return pData;
}

//
// liefert Laenge des DatenElements;
//
int
TMessage::getLength(const char* msg)
{
    int i=0;
    while (msg[i]!=0) 
    {
        if (msg[i]==limiter) return i;
        i++;
    }
    return i;
}  	  
