#ifndef __MSGDATA_HH
#define __MSGDATA_HH
//
//  Project   : besy
//  File      : msgdata.hh
//  Author    : Ronald Kriemann
//  Purpose   : class which holds all data for a msg for the bankserver
//

#include <stdio.h>
#include <string.h>
#include <iostream.h>

#include "TString.hh"
#include "TBitArray.hh"
#include "list.hh"

#include "../bank/permission.hh"


// positions for the different values in the bool-array
#define KUNDENNR      0
#define KUNDENPWD     1
#define KUNDENNAME    2
#define KONTONR       3
#define BETRAG        4
#define ZIELKONTONR   5
#define ZWECK         6
#define ZIELNAME      7
#define ZIELNUMMER    8
#define RECHTABHEBEN  9
#define RECHTKONTOAUSZUG  10
#define SERIALNO      11
#define COINSTR       12
#define COINIDS       13


enum Operation  { NOOP, LOGIN, ONELOGIN, KSTAND, KAUSZ,
                  UEBERW, RECHTE, RSA, ENDE, CHECKSERIAL, SIGNCOIN, SPENDCOIN };

//
//
//
class MSGData
{
protected:
	// Kundennummer, Kundenpassword
	int   nKundenNr;
	char  szKundenPwd[256];
	char  szKundenname[256];

	// Kontonummer
	int   nKontoNr;

	// Betrag, Zielkontonummer, Zweck, Zielname, Recht
	double fBetrag;
	int    nZielKontoNr;
	char   szZweck[256];
	char   szZielname[256];
	int    nZielnummer;
	int    nRechtAbheben;
    int    nRechtKontoauszug;

	Operation nOp;

	// this array holds flags, wich indicate 
	// if a value was set or not
	bool  bWasSet[13];
    
    int      serialNo;
    TString  coinStr;
    TString  coinIds;

public:
	// constructor
	MSGData ();

    // destructor
	~MSGData ();

	// methods for setting and getting the diff. values and
	// asking if they where set
	int  getKundenNr () const;
	void setKundenNr ( int nr );
	bool hasKundenNr () const;

	char* getKundenPwd ();
	void setKundenPwd ( const char* pwd );
	bool hasKundenPwd () const;


	char* getKundenname ();
	void setKundenname ( const char* name );
	bool hasKundenname () const;

	int  getKontoNr () const;
	void setKontoNr ( int nr );
	bool hasKontoNr () const;

	double getBetrag () const;
	void setBetrag ( double betrag );
	bool hasBetrag () const;

	int  getZielKontoNr () const;
	void setZielKontoNr ( int nr );
	bool hasZielKontoNr () const;

	char* getZweck ();
	void setZweck ( const char* zweck );
	bool hasZweck () const;

	char* getZielname ();
	void setZielname ( const char* name );
	bool hasZielname () const;
         
	int getZielnummer ();
	void setZielnummer ( int nummer );
    bool hasZielnummer () const;
  
	Operation getOperation () const;
	void setOperation ( Operation o );
    bool hasOperation () const;

	int getRechtAbheben () const;
	void setRechtAbheben (int value);
    bool hasRechtAbheben() const;

    int getRechtKontoauszug () const;
	void setRechtKontoauszug(int value);
    bool hasRechtKontoauszug() const;

    const int getSerialNo () const;
    void setSerialNo ( int s );
    const bool hasSerialNo () const;

    const TString & getCoinStr () const;
    void setCoinStr ( const TString & string );
    const bool hasCoinStr () const;

    const TString & getCoinIDs () const;
    void setCoinIDs ( const TString &arrID );
    const bool hasCoinIDs () const;
    
	// clears all information (except Name,Login,Ktnr)
	void clear ();

	// output
	friend ostream& operator << ( ostream& os, const MSGData& msgData );
};

#endif  // __MSGDATA_HH
