//
//  Project   : libmisc
//  File      : scanner.cc
//  Author    : Ronald Kriemann
//  Purpose   : class for a lex. scanner (returns predefined tokens)
//  Date      : 14.05.1997
//  Changes   : 04.06.1997 - removed bug in string-parsing
//

#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <iostream.h>

#include "scanner.hh"

// different types of data to scan
#define  NOTHING  0
#define  INT      1
#define  FLOAT    2
#define  STRING   3

//
// type for the string array
//
struct TScanString
{
	char  szName[32];
	Token token;
	uint  dataType;
};

// 
// the array with the strings to scan
//
TScanString  scanStrings[] =
{
	{"ZKTNR"      , tokZKTNR      , INT },
	{"KTNR"       , tokKTNR       , INT },
	{"ZNAME"      , tokZNAME      , STRING },
	{"ZNR"        , tokZNR        , INT },
	{"NAME"       , tokNAME       , STRING },
	{"KUNDENNR"   , tokNUMMER     , INT },
	{"PASSWORT"   , tokPASSWORT   , STRING },
	{"ONETIMEPW"  , tokONEPW      , STRING },
	{"BETRAG"     , tokBETRAG     , FLOAT },
	{"ZWECK"      , tokZWECK      , STRING },
	{"ENDE"       , tokENDE       , NOTHING },
	{"OP"         , tokOP         , NOTHING },
	{"KSTAND"     , tokOPKSTAND   , NOTHING },
	{"KAUSZ"      , tokOPKAUSZ    , NOTHING },
	{"UEBERW"     , tokOPUEBERW   , STRING },	
	{"RECHTE"     , tokOPRECHTE   , NOTHING },
	{"ABHEBEN"    , tokABHEBEN    , STRING },
	{"KONTOAUSZUG", tokKONTOAUSZUG, STRING },
    {"SIGNCOIN"   , tokOPSignCoin , STRING },
};

uint noOfScanStr = 19;

//
// constructor
//
Scanner::Scanner ( const TString & string ) 
{
    szString = string;
}

//
// destructor
//
Scanner::~Scanner ()
{
}

//
// returns next token and sets value of token
//
int 
Scanner::nextToken( TokenData& data ) 
{
	uint i;

	while (szString.length() > 0)
	{
		switch (szString[0])
		{
            case '.':
		        // detect '.' at the beginning of a line
                szString <<= 1;
                data.token = tokDot;
                return SUCCESS;
                break;
		  
            case '+':
                // first check, what comes next (num or alpha)
                // (we assume that there is something)
                if (isdigit(szString[1]) == 0)
                {
                    szString <<= 1;
                    data.token = tokADD;
                    return SUCCESS;
                }// if
                break;

            case '-':
                // first check, what comes next (num or alpha)
                // (we assume that there is something)
                if (isdigit(szString[1]) == 0)
                {
                    szString <<= 1;
                    data.token = tokSUB;
                    return SUCCESS;
                }// if

            case '0': case '1': case '2': case '3':
            case '4': case '5': case '6': case '7':
            case '8': case '9':
            {
                int    nInt;
                TFloat fFloat;

                // a number follows
                szString >> nInt;
                
                if (szString[0] == '.')
                {
                    szString <<= 1;

                    if (isdigit( szString[1] ) == 0)
                        return ERROR;
                    
                    // it is a float
                    data.token  = tokFloat;
                    data.fFloat = nInt;

                    szString >> nInt;
                    fFloat = nInt;
                        
                    while (fFloat >= 1)
                        fFloat /= 10;
                        
                    if (data.fFloat >= 0)
                        data.fFloat += fFloat;
                    else
                        data.fFloat -= fFloat;
                        
                    return SUCCESS;
                }// if
                else
                {
                    // it is a int
                    data.token = tokInt;
                    data.nInt  = nInt;
                    return SUCCESS;
                }// else
            }
            break;

            case '#':
            case '\n':
            case 13:
            case 0:
                // end of line
                szString = "";
                data.token = tokEOL;
                return SUCCESS;

            case ' ': 
            case '\t':
                // eat it up
                break;			

            default: // more complicated strings
                for (i = 0; i < noOfScanStr; i++)
                {
                    if (szString.strstr( scanStrings[i].szName ).length() > 0)
                    {
                        szString <<= strlen( scanStrings[i].szName );
                        data.token = scanStrings[i].token;

                        if (data.token == tokOPSignCoin)
                        {
                            szString <<= 1;

                            szString.copyTo( data.szString, sizeof(data.szString) );
                            szString = "";
                        }// if
                        else
                        {
                            ulong oldLength = szString.length();
                            
                            switch (scanStrings[i].dataType)
                            {
                                case INT    :
                                    szString >> data.nInt;
                                    if (oldLength == szString.length())
                                        return ERROR;
                                    break;
                                
                                case FLOAT  :
                                    szString >> data.fFloat;
                                    if (oldLength == szString.length())
                                        return ERROR;
                                    break;
                                
                                case STRING :
                                    getString( data.szString );
                                    break;
                                
                                case NOTHING:
                                default     :
                                    break;
                            }// switch
                        }// else
                    
                        return SUCCESS;
                    }// if
                }// for

                // found nothing known
                data.token = tokNothing;
                szString = "";
                getString( data.szString );
                return SUCCESS;

                break; 
		}// switch
        
		// next char
		szString <<= 1;
	}// while

	// end of line
	data.token = tokEOL;
    
	return SUCCESS;
}

//
// tries to read an integer and gives it back
//
int 
Scanner::getInt( int &nInt ) 
{
	bool noNumberYet = TRUE;
	bool isNegativ   = FALSE;

	nInt = 0;
	// scan for an int
	while (szString[nPos] != 0)
	{
		if (isdigit(szString[nPos]) != 0)
		{
			// a number
			noNumberYet = FALSE;
			if (isNegativ == FALSE)
				nInt = nInt * 10 + alpha2digit(szString[nPos]);
			else
				nInt = nInt * 10 - alpha2digit(szString[nPos]);
		} else if (isspace(szString[nPos]) != 0)
		{
			// we got a number
			if (noNumberYet == FALSE)
				return SUCCESS;
		} else if (szString[nPos] == '-')
		{
			// we got a number
			if (noNumberYet == FALSE)
				return SUCCESS;
			else // a minus -> invert the value
				isNegativ = !isNegativ;
		} else
		{
			// we got a number
			if (noNumberYet == FALSE)
				return SUCCESS;
			else // error
				return ERROR;
		}

		// next char
		nPos++;
	}// while
	// we got a number
	if (noNumberYet == FALSE)
		return SUCCESS;
	else
		return ERROR;
}

//
// tries to read an string, without EOL-char or other spaces on the end or begin
//
void
Scanner::getString ( char *string ) 
{
    TString  buffer;
    
	// remove leading spaces
	while ((szString.length() > 0) &&
           (szString[0] != 0) &&
           (isspace(szString[0]) != 0))
    {
		szString <<= 1;
    }// while

	buffer = szString;

	// remove spaces on the end
	while ((buffer.length() > 0) &&
           ((buffer[buffer.length()-1] == 0) ||
            (isspace(buffer[buffer.length()-1]) != 0)))
    {
        buffer <<= 1;
	}// while
    
    buffer.copyTo( string, 16384 );
}

//
// converts alpha char in digit
//
int
Scanner::alpha2digit ( char alpha ) 
{
    switch (alpha)
    {
        case '0' : return 0;
        case '1' : return 1;
        case '2' : return 2;
        case '3' : return 3;
        case '4' : return 4;
        case '5' : return 5;
        case '6' : return 6;
        case '7' : return 7;
        case '8' : return 8;
        case '9' : return 9;
        default  : return -1;
    }// switch
    
    return -1;
}
