#include <stdio.h>
#include "socket.hh"
#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>
#include <netdb.h>
#include <sys/time.h>
#include <string.h>


TSocket::TSocket() 
{  
	bconnected=false;
	bserver=false;
	len  = sizeof(struct sockaddr_in);
	pSocketTimer = new (struct timeval);
}


TSocket::TSocket(int portnumber) 
{
	nport=portnumber;
	bconnected=false;
	bserver=false;
	len  = sizeof(struct sockaddr_in);
	pSocketTimer = new (struct timeval);
}

TSocket::~TSocket() 
{
	if (bconnected) ::close(nsd);
	if (bserver) ::close(serverSocket);
	delete pSocketTimer;
}

//Set the portnumber
void
TSocket::setPort(int port)
{
	nport=port;
}


//
//Creating the  Serversocket (ONLY For the Server)
//
int
TSocket::create()
{
	long n;
	char szHostname[128];  

	//Checking localhost
	if (gethostname(szHostname, sizeof(szHostname)) < 0) 
    {
		cerr << "TSocket::create(): ERROR detecting localhost" << endl;
		return ERROR;
    }

	// create the address of the socket
	memset( &socketAddr, 0, sizeof( struct sockaddr_in ));
	socketAddr.sin_family = AF_INET;
	socketAddr.sin_port   = htons(nport);
	n = INADDR_ANY;
	memcpy(&socketAddr.sin_addr, &n, sizeof(long));

	// create socket
	if ( (serverSocket = socket( AF_INET, SOCK_STREAM, 0)) < 0) 
    {
		cerr << "TSocket::create(): ERROR couldn't create socket" << endl;
		return ERROR;
    }
	// bind the socket to the adress
	if (bind(serverSocket, (struct sockaddr *) &socketAddr, sizeof(struct sockaddr_in)) < 0)
    {
		cerr << "TSocket::create(): ERROR couldn't bind socket" << endl;
		return ERROR;
    }

	// enabling listen for a connection
	if (listen(serverSocket, 5) < 0)
    {
		cerr << "TSocket::create(): ERROR couldn't listen to socket" << endl;
		return ERROR;
    }    

	bserver=true;
	return SUCCESS;
}

//
//Waiting for Clients (ONLY for the server)
//Returning Socketdescriptor for communication
//if it fails the return is -1
//
int
TSocket::select()
{
// 	FD_ZERO( &readySockets );
// 	FD_SET( serverSocket, &readySockets );

// 	pSocketTimer->tv_sec = 0;
// 	pSocketTimer->tv_usec = 0;

// 	// check if clients have connected
// 	if (::select( serverSocket+1, &readySockets, 0, 0, pSocketTimer ) < 0)
// 	{
// 		perror( "select" );
// 		return -1;
// 	}
// 	else if (FD_ISSET(serverSocket, &readySockets)) 
// 	{
// 		cout << "trying to accept" << endl;

// 		// a client has connected to our server
// 		// accept the connection
// 		// define the socketdescriptor for communication

	if ((nsd = accept(serverSocket, (struct sockaddr *) &socketAddr, &len)) < 0)
	{
		cerr << "TSocket::select(): ERROR doing accept" << endl;
		return -1;
	}// if
	else
		return nsd;

// 	}

// 	return -1;
}


//
//Connecting to a server (ONLY for the Client)
//Returning socketdescriptor for the communication
//
int
TSocket::connect(TString host)
{
	struct hostent *pHostent;
	char szHostname[128];  
	//Checking localhost
	if (gethostname(szHostname, sizeof(szHostname)) < 0) 
    {
		cerr << "TSocket::connect(): ERROR detecting localhost" << endl;
		return false;
    }
  
	//Checking Server
	host.copyTo(szHostname);szHostname[host.length()]=0;  
	if ((pHostent = gethostbyname(szHostname)) == NULL) 
    {
		cerr << "TSocket::connect(): ERROR server not found" << endl; 
		return -1;
    }
	// create the address of the socket
	memset( &socketAddr, 0, sizeof( struct sockaddr_in ));  
	socketAddr.sin_family = AF_INET;
	socketAddr.sin_port = htons(nport);
	memcpy( &socketAddr.sin_addr, pHostent->h_addr_list[0], pHostent->h_length );  
  
	// create socket
	if ( (nsd = socket( AF_INET, SOCK_STREAM, 0)) < 0) 
    {
		cerr << "TSocket::connect(): ERROR couldn't create socket" << endl;
		return -1;      
    }
	// connect to server
	if (::connect(nsd,(struct sockaddr *) &socketAddr, sizeof(socketAddr)) < 0 )
    {
		cerr << "TSocket::connect(): ERROR couldn't connect to server" << endl; 
		return -1;
    }  
	bconnected=true;
	return nsd;
}


bool 
TSocket::connected()
{ return bconnected; }

//closes the communicationsocket (ONLY for the client)
bool 
TSocket::close()
{
	if (bconnected) return (::close(nsd)==0);
	return false;
}

//closes the socket sd
bool 
TSocket::close(int sd)
{
	return (::close(sd)==0);
}


//Send a String through the socket sd
bool 
TSocket::send(int sd, TString string)
{
    int   size;
	char  socketstring[string.length()+1];
	int   written_bytes = 0;
	char  buffer[BUFFERSIZE+1];
	int   n = 0;
	int   message_size;
	bool  bERROR=false;
    
	string.copyTo( socketstring, string.length()+1 );
	socketstring[string.length()] = 0;
    
    message_size = strlen(socketstring);
    
	if  (strlen(socketstring) > BUFFERSIZE)
        cout << "TSocket::send(): buffer overrun\n" ;
    
	while ( (written_bytes < message_size) || bERROR) 
    {
		bzero((char *) &buffer, sizeof(buffer));

        size = 0;
        
		for (int i=1; i<=BUFFERSIZE;i++)
        {
			if (i+written_bytes <= message_size)
			{
				buffer[i-1]=socketstring[written_bytes+i-1];	  
				size++;
			}// if
        }// for

//        cout << "TSocket::send (...) : length = " << size << endl; 
//         cout << "TSocket::send (...) :    msg = " << buffer << endl;

		if ((n=::send(sd,buffer,size,0)) < 0)	
		{
			cerr << "TSocket::send(): ERROR while sending" << endl;	         
			bERROR=true;
		}// if

		written_bytes +=n;      
    }// while
    
	//	cout << written_bytes << "," << message_size << endl;
	return written_bytes >= message_size;
}

bool
TSocket::sendln(int nsd,TString string)
{  
	if (string.length() > 0) string+="\n";
	send(nsd,string);  
}


bool
TSocket::send(TString string)
{
	send(nsd,string);
}

bool
TSocket::sendln(TString string)
{
	sendln(nsd,string);
}



//Receives a string through the Socket
//Returning received Bytes
int
TSocket::recv(int sd, TString &Result)
{
	char buffer[BUFFERSIZE+1];
	int  n;

	memset(buffer, 0, sizeof(buffer));

	if ((n = ::recv(sd, buffer, sizeof(buffer), 0)) > 0)
    {                 
		Result.copyFrom(buffer, n);
//        cout << "TSocket::recv (...) : length = " << n << endl; 
//        cout << "TSocket::recv (...) :    msg = " << buffer << endl;
		return Result.length();
    }// if

	return n;
}

//Receives a string through the Socket
//Returning received Bytes
int
TSocket::recv(TString &Result)
{
	return recv(nsd,Result); 
}



//Receives a line\n from Socket sd
int
TSocket::recvln(int sd, TString &Result)
{
	TString Buffer;
	int     n=0;
	bool    bEOL=false;

	Result.setLength(0);

	// read until zero-length or \n 
	while ( !bEOL && ((n = recv(sd,Buffer)) > 0) )    
    {      
		//		cout << "recvl:[" << Buffer << "{" << n << "}" << endl;
		if (Buffer.length() > 0) 
			bEOL = ('\n' == Buffer[Buffer.length()-1] );

// 		if (Result.length()==0) 
// 			Result  = Buffer; 
// 		else 
// 			Result += Buffer;
        Result = Buffer;
    } 
	return n;
}

//Receives a line\n from Communication-Socket (ONLY for Client)
int
TSocket::recvln(TString &Result)
{
	return recvln(nsd,Result);
}
